function [hOut, img] = fakeaxescontent(ax, cmd)
% FAKEAXESCONTENT  replace AXES content with a non-compressed image of itself
% 
% Useful if you have a lot of little dots that will take up too much space
% in a PostScript file, but hardly any space if re-represented as an image
% before exporting to PostScript.
% 
% Tip: when distilling the PostScript, turn off downsampling/subsampling,
% but turn on compression (automatic, maximum).
% 
% FAKEAXESCONTENT by itself operates on GCA
% FAKEAXESCONTENT(H) operates on all AXES objects in the hierarchy starting
% with object H (including hidden AXES objects).
% 
% FAKEAXESCONTENT(H, CMD) executes the string CMD in the caller workspace
% as soon as the new (invisible) AXES have been created.

% Part of PFCMP version 2.5.41, an extension to the PSIGNIFIT toolbox for MATLAB versions 5 and up.
% The license and NO WARRANTY statement for PSIGNIFIT also applies to this release - see psych_legal.m
% Copyright (c) J.Hill 2002.
% mailto:psignifit@bootstrap-software.org
% http://www.bootstrap-software.org/psignifit/

if nargin < 1, ax = gca; end

ax = findall(ax, 'type', 'axes');
if length(ax) ~= 1
	for i = 1:length(ax)
		h(i, 1) = fakeaxescontent(ax(i));
	end
	if nargout, hOut = h; img = {}; end
	return
end

p = gettrueaxesposition(ax, 'pixels');

%% the following section rounds the /original/ figure and axes positions to integer numbers of pixels
p = round(p);
fig = get(ax, 'parent');
oldu = get(ax, 'units');
set(fig, 'units', 'pixels')
set(fig, 'position', round(get(fig, 'position')))
set(fig, 'units', oldu)
oldu = get(ax, 'units');
set(ax, 'units', 'pixels')
set(ax, 'position', p)
set(ax, 'units', oldu)
%%

getprefs('oversample', 1)
imagesize = round(p(3:4) .* oversample);
newfig = figure('menubar', 'none', 'units', 'pixels', 'resize', 'off', 'position', [1 1 imagesize], 'visible', 'off');
set(newfig, 'position', [1 1 imagesize]) % repetition = workaround for MathWorks' latest annoying implementation bug (Matlab 6.1.0.450) 
set(newfig, 'colormap', get(get(ax, 'parent'), 'colormap'))
newax = copyobj(ax, newfig);
set(findall(newax), 'deletefcn', '') % no side effects of deletion of copy please
axpos = [1 1 imagesize];
set(newax, 'units', 'pixels', 'position', axpos, 'visible', 'off')

if nargin < 2, cmd = ''; end
if ~isempty(cmd), evalin('caller', cmd); end

filename = [tempname '.tif'];
printfig(newfig, filename, 'format', 'tiffnocompression')
delete(allchild(newax))
img = imread(filename);
delete(filename)

xl = get(ax, 'xlim');
yl = get(ax, 'ylim');
xdir = get(ax, 'xdir');
xrev = strcmp(xdir, 'reverse');
ydir = get(ax, 'ydir');
yrev = strcmp(ydir, 'reverse');

[iysiz ixsiz ans] = size(img);
ix = linspace(xl(1), xl(2), ixsiz+1);
ix = ix(1:end-1) + diff(ix)/2;
iy = linspace(yl(1), yl(2), iysiz+1);
iy = iy(1:end-1) + diff(iy)/2;

if xrev, img = img(:, end:-1:1, :); end
if ~yrev, img = img(end:-1:1, :, :); end
h = image(ix, iy, img);

delete(get(ax, 'children'))
h = copyobj(h, ax);
set(ax, 'layer', 'top')
delete(newfig)
if nargout, hOut = h; end
