function out = latex(varargin)
% LATEX(STR1, [STR2, ...]) % strings
%    Makes struct from strings, or struct('string', STR1) if only one.
%    Auto-expands field names  'xticklabel', 'yticklabel' and 'zticklabel'.
%    Standardizes the case of field names to lower case.
%    Returns the struct.
% LATEX(H, S) % where S is a 1-by-1 struct
%    Standardizes the case of the field names of S to lower case.
%    Checks that field names are appropriate for the handle type of H.
%    Adds the fields of S to the 'latex' substruct of the userdata of H.
%    Returns H if asked.
% LATEX(H, STR1, [STR2...]) % strings
%    Equivalent to LATEX(H, LATEX(STR1, STR2...)).
% LATEX(H)
%    Returns the 'latex' field of the userdata of H, if any.
% 
% Used to set LaTeX alternatives to the string content of TEXT objects
% or axes tick labels. e.g:
%
%       h = title('Results for \theta_{gen}');
%       % (crude MATLAB TeX)
%       alt = 'Results for $\bm{\hat{\theta}}_{\mbox{\tiny gen}}$';
%       % (much better)
%       latex(h, alt)  % could also say latex(h, 'string', alt)
% 
%       % To do the ticklabels:
%       latex(gca, 'xticklabel', xticklatex, 'yticklabel', ....)
%       % (property names may be abbreviated to 'x', 'y' and 'z')
% 
% Also used with figures, to set the dimensions of the figure and the
% scaling of font sizes (see LATEXPORT for details). e.g:
% 
%       latex(gcf, 'width', '12cm', 'fontFamily', 'cmss')
% 
% The LaTeX alternatives are picked up when exporting with the
% function LATEXPORT.

% Part of PFCMP version 2.5.41, an extension to the PSIGNIFIT toolbox for MATLAB versions 5 and up.
% The license and NO WARRANTY statement for PSIGNIFIT also applies to this release - see psych_legal.m
% Copyright (c) J.Hill 2002.
% mailto:psignifit@bootstrap-software.org
% http://www.bootstrap-software.org/psignifit/

if nargin < 1, error('no input arguments'), return, end
h = varargin{1};
if isnumeric(h)
	varargin(1) = [];
	if nargout, out = h; end
	if isempty(varargin)
		if ~all(ishandle(h(:))) | prod(size(h)) ~= 1, error('H must be a scalar handle'), end
		out = get(h, 'userdata'); if isfield(out, 'latex'), out = out.latex; else out = []; end
		return
	end
	if length(varargin) == 1 & iscellstr(varargin), varargin = [{'string'} varargin]; end
	if length(varargin) > 1 & rem(length(varargin), 2) == 0
		try, latex(h, latex(varargin{:})), catch, error(lasterr), end
		return
	end
	s = varargin{1};
	varargin(1) = [];
	if ~isempty(varargin)
		error('invalid input argument format')
	end
	if ~all(ishandle(h(:))) | prod(size(h)) ~= 1, error('H must be a scalar handle'), end
	if ~isequal(s, []) & (~isstruct(s) | prod(size(s)) ~= 1), error('S must be a 1-by-1 struct'), end
	if isstruct(s), f = lower(fieldnames(s)); v = struct2cell(s); else, f = {}; v = {}; end
	u = get(h, 'userdata');
	if ~isstruct(u) | prod(size(u)) ~= 1, u = []; end
	if isfield(u, 'latex'), l = u.latex; else, l = []; end
	if ~isstruct(l) | prod(size(l)) ~= 1, l = []; end
	if isempty(s), l = []; end % so LATEX(H, []) removes the LATEX struct (whereas anything else---struct, strings, etc---would be additive)
	switch lower(get(h, 'type'));
		case 'text'
			if any(~match(f, 'string'))
				error('for TEXT objects, only the ''String'' property can be set')
			end
			if ~iscellstr(v), error('for TEXT objects, property values must be strings'), end
		case 'axes'
			legal = {'XTickLabel', 'YTickLabel', 'ZTickLabel'};
			if any(~match(f, lower(legal)))
				legal(2, :) = {', '}; legal = [legal{1:end-1}];
				error(sprintf('for AXES objects, only the following properties can be set:\n\t%s', legal))
			end
			if ~iscellstr(v), error('for AXES objects, property values must be strings'), end
		case 'figure'
			legal = {'Width', 'Height', 'BaseFontSizeOnScreen', 'BaseFontSizeOnPaper', 'FontScaling', 'FontFamily', 'MarkerScaling'};
			if any(~match(f, lower(legal)))
				legal(2, :) = {', '}; legal = [legal{1:end-1}];
				error(sprintf('for FIGURE objects, only the following properties can be set:\n\t%s', legal))
			end
			for i = 1:length(v)
				val = v{i}; 
				if isempty(val)
					v{i} = [];
				elseif ~isstr(val)
					if ~isa(val, 'double') | prod(size(val)) ~= 1, val = inf; end
					if ~isreal(val) | isinf(val) | isnan(val) | val <= 0, error('for FIGURE objects, property values must be real positive scalars'), end
				end
			end
		otherwise, error('H must be an AXES or TEXT object handle')
	end
	for i = 1:length(f), l = setfield(l, f{i}, v{i}); end
	if ~isempty(l)
		u.latex = l;
	elseif isfield(u, 'latex')
		u = rmfield(u, 'latex'); 
	end
	set(h, 'userdata', u)
	return
end

if rem(length(varargin), 2), error('if more than one string is supplied, they must come in key/value pairs'), end
keys = lower(varargin(1:2:end));
vals = varargin(2:2:end);
for i = 1:length(keys)
	if ~isempty(keys{i}) & strmatch(keys{i}, 'xticklabel'), keys{i} = 'xticklabel'; end
	if ~isempty(keys{i}) & strmatch(keys{i}, 'yticklabel'), keys{i} = 'yticklabel'; end
	if ~isempty(keys{i}) & strmatch(keys{i}, 'zticklabel'), keys{i} = 'zticklabel'; end
end
s = [keys;vals];
try, out = struct(s{:}); catch, error(lasterr), end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function m = match(str, legal)

str = cellstr(str);
legal = cellstr(legal);
m = logical(zeros(size(str)));
for i = 1:prod(size(str))
	m(i) = any(strcmp(str{i}, legal));
end
