function tickfmt(varargin)
% TICKFMT(FMT)
% 	formats tick labels according to the SPRINTF format FMT.
% TICKFMT(H, ...)
% 	operates on all axes objects in the hierarchy that starts
% 	with object H.
% TICKFMT('XZ', ...) % for example
% 	operates only on x and z axes (or whatever is specified).
% TICKFMT
% 	on its own smartens the labels up 'intelligently': they
% 	are all expressed to the same number of decimal places
% 	where possible, and a + sign is prepended to positive
% 	values iff there are also negative values on the axis.
% 
% Aesthetic feature: 0 always appears unsigned, even if a
% '%+' format was requested.

% Part of PFCMP version 2.5.41, an extension to the PSIGNIFIT toolbox for MATLAB versions 5 and up.
% The license and NO WARRANTY statement for PSIGNIFIT also applies to this release - see psych_legal.m
% Copyright (c) J.Hill 2002.
% mailto:psignifit@bootstrap-software.org
% http://www.bootstrap-software.org/psignifit/

axh = '';
whichAx = '';
fmt = '';

if length(varargin) > 3, error('too many arguments'), end
for i = 1:length(varargin)
	arg = varargin{i}; err = 1;
	if isempty(arg)
		err = 0;
	elseif isnumeric(arg)
		if isempty(axh)
			axh = arg; err = 0;
		end
	elseif isstr(arg)
		if isempty(whichAx) & ~isempty(arg) & all(ismember(lower(arg), 'xyz'))
			whichAx = lower(arg); err = 0;
		elseif isempty(fmt)
			fmt = arg; err = 0;
		end
	end
	if err, error(sprintf('could not understand argument #%d', i)), end
end
if isstr(axh), axh = gca; end
axh = findobj(axh, 'type', 'axes');
if isempty(whichAx), whichAx = 'xyz'; end

for i = 1:prod(size(axh))
	h = axh(i);
	for w = whichAx(:)'
		if isempty(fmt)
			n = [];
			t = cellstr(get(h, [w 'ticklabel']));
			for j = 1:length(t), n = [n; str2num(t{j})]; end
			ndp = 0; while ndp <= 10 & any(n - floor(n) > abs(n) * 1e-10), n = n * 10; ndp = ndp + 1; end
			if min(n) < 0 & max (n) > 0, f = '%+'; else f = '%'; end
			if ndp == 0, f = [f 'g']; else f = sprintf('%s.%df', f, ndp); end
		else
			f = fmt;
		end
		mode = get(h, [w 'ticklabelmode']);
		if strcmp(lower(mode), 'auto')
			t = get(h, [w 'tick']);
			t = num2cell(t(:));
		else
			t = cellstr(get(h, [w 'ticklabel']));
			for j = 1:length(t)
				n = str2num(t{j});
				if ~isempty(n), t{j} = n; end
			end
		end
		for j = 1:length(t)
			if isnumeric(t{j})
				n = t{j};
				t{j} = sprintf(f, n);
				n = str2num(t{j}); % this apparently redundant step is because 0 doesn't always mean 0 (<<sigh>>)
				if ~isempty(n)
				% 	option 1 - just remove the + or - sign from 0
					if n == 0 & ismember(t{j}(1), '+-'), t{j}(1) = []; end
				%	option 2 - remove decimal places as well
					if isempty(fmt) & n == 0, t{j} = '0'; end
				end
			end
		end
		set(h, [w 'ticklabel'], t)
	end
end
